/***********************************************************************************
************************************************************************************

PIC10(L)F322 Development Board (AC103011-P) PWM Control

Copyright (C)2012 Giovanni Bernardo
http://www.settorezero.com
date:  10/10/12

MCU:					Microchip PIC10F322
Clock:					16MHz Internal
Compiler:				HI-TECH C Compiler for PIC10/12/16 MCUs Version 9.83

You may NOT distribute or sell this code without explicit authorization

PER PIACERE: NON UTILIZZATE CODICE E SCHEMA SU ALTRI SITI SE
VOLETE CHE SETTOREZERO CONTINUI A RIMANERE ATTIVO!

PER PIACERE! GRAZIE!

THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
WE ARE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
(INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.

************************************************************************************
************************************************************************************/

#include <htc.h>
#define _XTAL_FREQ	16000000UL

#define	SW1		PORTA3
#define LED1	LATA1
#define LED2	LATA0
// trimmer on 	PORTA2

// pwm control parameters
int pwmval=0; 				// actual pwm value
unsigned char direction=0;	// pwm scan 1:(255->0) 0:(0->255)
int pwmdelay=0;				// pwmdelay = pwmcounter, when pwmdelay=0 pwm value is changed
int pwmcounter=50;			// pwmdelay determined by trimmer
int oldpwmcounter=50;		// previous value of pwm delay

// ad conversion parameters
unsigned char advalue;		// actual value from AD conversion
char means=0;				// actual number of means
unsigned int adm=0;			// ad value sum
unsigned char adman=0;		// ad value for manual mode

char mode=0;				// mode (auto/man), determined by SW1

#define NMEANS		32			// number of means to do
#define NMEANSDIV	6		// 2^(NMEANSDIV) = NMEANS
#define MAXPWM		180
#define MINPWM		0
#define MODE_MAN	0
#define MODE_AUTO	1
#define MINDELAY	1

// internal oscillator used
__CONFIG(FOSC_INTOSC & BOREN_OFF & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & LVP_OFF & LPBOR_OFF & BORV_LO & WRT_OFF);

void main(void)
	{
	OSCCON=0b01110000; // internal oscillator @ 16MHz
	while (!HFIOFS); // wait for internal oscillator stability
	
	TRISA =0b00001101; // A1 output. A2,A3 inputs (A2=analog). A0 input for now, will become output later, after PWM settings
	ANSELA=0b00000100; // only A2 as analog input
	ADCON =0b11101001; // adc enabled, read from AN2, clock=frc
	
	// initial values
	pwmdelay=pwmcounter;
	pwmval=MAXPWM;
	direction=1;
	mode=MODE_AUTO;
	LED1=MODE_AUTO;

	// TOsc=62.5nS (FOsc=16MHz)
	// prescaler 1:64 -> tmr0 increments by 1 every 62.5*4*64 = 16uS
	OPTION_REG=0b10000101; // prescaler 1:64 assigned to timer0
	TMR0=256-62; // 16*62 = 992uS. Overflow interrupt will fire every 992uS
	
	// PWM module setting (See datasheet, pag. 110)
	PWM1CON=0; // reset PWM1 control register
	PR2=0x80; // pwm period = 32.25uS (PWM frequency=31KHz). See datasheet, pag.108
	PWM1DCH=0; // reset PWM1 duty cycle register H
	PWM1DCL=0; // reset PWM1 duty cycle register L
	TMR2IF=0; // timer2 interrupt flag reset
	T2CON=0b00000000; // timer2 prescaler 1:1
	TMR2ON=1; // enable timer2
	PWM1OE=1; // enable PWM1 output
	while (!TMR2IF); // wait until timer2 overflows
	PWM1EN=1; // enable PWM1 module
	TRISA &= 0b11111110; // enable only PORTA0 (PWM1 out) as output
	
	GIE=1; // global interrupt enable
	TMR0IE=1; // timer0 interrupt enable
		
	while (1)
		{
		GO_nDONE=1; // start AD conversion
		while(GO_nDONE); // wait until conversion complete
		advalue=ADRES; // AD conversion result
		
		// number of means not reached: sum the advalues
		if (means<NMEANS)
			{
			adm+=advalue;
			means++;
			}
		else // number of means reached
			{
			means=0;
			adman=adm>>NMEANSDIV; // ad value if manual mode is selected
			
			pwmcounter=adman>>3; // report 0-255 to 0-31 (for delay: 255 is too much for me!)
			adm=0; // reset sum			

			if (pwmcounter<MINDELAY)
				{
				pwmcounter=MINDELAY;
				}

			// reset cycle if new value is different from previous value
			if (pwmcounter != oldpwmcounter)
				{
				pwmdelay=pwmcounter;	
				}
			
			// remember old value
			oldpwmcounter=pwmcounter;
			}
			
		// check SW1
		if (!SW1) // SW1 pressed?
			{
			while(!SW1); // still pressed?
			
			// invert mode/LED
			mode ^= 1;
			LED1 ^= 1;
			// if auto mode selected, reset cycle
			if (mode==MODE_AUTO)
				{
				pwmval=MAXPWM;
				direction=1;
				pwmdelay=20; // dummy delay
				}
			} // \sw1 pressed
		} // \while(1)
	} // \main

// tmr0 interrupt every 992uS
void interrupt ISR(void)
	{
	TMR0IF=0; // don't forget to reset the interrupt flag!
	if (mode==MODE_AUTO) // auto mode : pulse mode, trimmer determines the delay
		{
		pwmdelay--;
		if (pwmdelay==0)
			{
			pwmdelay=pwmcounter; // reload counter
			if (direction==1) // direction 1 : 255 -> 0
				{
				pwmval--;
				}
			else // direction 0 : 0 -> 255
				{
				pwmval++;
				}
			
			if (pwmval==MINPWM)
				{
				direction=0;
				}
			else if (pwmval==MAXPWM)
				{
				direction=1;
				}
			}
		} // mode auto
		
	else // manual mode = pwm value determined by trimmer
	
		{
		pwmval=adman;
		}
	
	PWM1DCH=pwmval;
	TMR0=256-62; // reload timer0
	} // \isr
